/****************************************************************************

Interface to the USB/CAN Interface device (LibUSB-Win32 driver)
(dynamic linkage)

RADStudio XE7

(c) 2020 Reusch Elektronik

Created: 2020-02-02
Changed: 2020-03-26

****************************************************************************/

//---------------------------------------------------------------------------

#pragma hdrstop

#include "uci_if.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TUCI::TUCI(void)
//---------------------------------------------------------------------------
// constructor
{
	FVendorID = USBD_VID;
	FProductID = USBD_PID;
}

//---------------------------------------------------------------------------
__fastcall TUCI::~TUCI(void)
//---------------------------------------------------------------------------
// destructor
{
	Close();
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::Open(AnsiString SerialNo)
//---------------------------------------------------------------------------
// open the communication to a device
{
	bool r = false;
	if (TUSBDevice::Open(SerialNo)) {
		TCommand Command = {USB_OPEN, 0};
		bool Ok = Write((char *)&Command,sizeof(TCommand),10)==sizeof(TCommand);
		if (Ok) {	// response from the device
			Sleep(1);
			if (Read((char *)&Command,sizeof(TCommand),10)==sizeof(TCommand)) {
				Ok = Command.param>0;	// result
			}
		}
		if (!Ok) TUSBDevice::Close();  // device could be opened, but didn't response -> close communication
		r = Ok;
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TUCI::Close(void)
//---------------------------------------------------------------------------
// close device communication
{
	if (FOpened) {
		TCommand Command = {USB_CLOSE, 0};
		Write((char *)&Command,sizeof(Command),10);
		TUSBDevice::Close();
	}
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::Status(TStatus *Status)
//---------------------------------------------------------------------------
// status request
// Status is filled with information, if result is true.
{
	bool r = false;
	TCommand Command = {USB_STATUS, 0};

	if (FOpened) {
		if (Write((char *)&Command,sizeof(TCommand),10)) {
			Sleep(1);
			if (Read((char *)Status,sizeof(TStatus),100)==(signed)sizeof(TStatus)) r = true;
		}
	} else Status->Status = STATUS_READERROR;
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::ClearStatus(bool Rx, bool Tx, bool ErrorCounters)
//---------------------------------------------------------------------------
// reset status and FIFO buffers
// Status zurck setzen
// Rx: true = clear receive buffer
// Tx: true = clear transmission buffer
// ErrorCounters: true = reset error counters
{
	bool r = false;
	TCommand Command = {USB_CLR_STATUS, 0};
	if (Rx) Command.param = 0x01;
	if (Tx) Command.param |= 0x02;
	if (FOpened) {
		r = Write((char *)&Command,sizeof(TCommand),10)==sizeof(TCommand);
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::DeviceInfo(TDeviceInfo *di)
//---------------------------------------------------------------------------
// request device information
// di is filled with information, if result is true.
{
	bool r = false;
	TCommand Command = {USB_INFO, 0};
	if (FOpened) {
		if (Write((char *)&Command,sizeof(Command),10)) {
			Sleep(1);
			if (Read((char *)di,sizeof(TDeviceInfo),100)==sizeof(TDeviceInfo)) r = true;
		}
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::GetDeviceConfig(TDeviceConfig *dc)
//---------------------------------------------------------------------------
// request device settings
// dc is filled with information, if result is true.
{
	bool r = false;
	TCommand Command = {USB_CONFIG_R, 0};
	if (FOpened) {
		if (Write((char *)&Command,sizeof(Command),10)) {
			Sleep(1);
			if (Read((char *)dc,sizeof(TDeviceConfig),100)==sizeof(TDeviceConfig)) r = true;
		}
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::SetDeviceConfig(TDeviceConfig *dc)
//---------------------------------------------------------------------------
// set new device settings
// dc has to contain valid device settings
// result is true, if settings successfully transferred to device
{
	bool r = false;
	dc->cmd = USB_CONFIG_W;
	if (FOpened) {
		r = Write((char *)dc,sizeof(TDeviceConfig),10)==sizeof(TDeviceConfig);
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::CAN_Rx(TCANData *cd)
//---------------------------------------------------------------------------
// retrieve received CAN messages
// if the result is true, cd is filled with valid data
// Note: On the result "false" no data is available. cd will be filled with
//       invalid data!
{
	bool r = false;
	TCommand Command = {USB_CAN_RX, 0};
	if (FOpened) {
		if (Write((char *)&Command,sizeof(Command),10)) {
			if (Read((char *)cd,sizeof(TCANData),100)==sizeof(TCANData)) r = cd->IdType!=CAN_ID_INVALID;
		}
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::CAN_Tx(TCANData *cd)
//---------------------------------------------------------------------------
// transmit data
// cd has to be filled with data to transfer to the device
// result is true, if the data is set successfully into the transfer FIFO
// buffer of the device
{
	bool r = false;
	TAcknowledge a;
	cd->cmd = USB_CAN_TX;
	if (FOpened) {
		r = Write((char *)cd,sizeof(TCANData),10)==sizeof(TCANData);
		if (r) {
			r = Read((char *)&a,sizeof(a),100)==sizeof(a);
			if (r) r = a.ack==ACK_OK;  // evaluation of the device response
		}
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::CAN_SetFilter(TCANFilter *cf)
//---------------------------------------------------------------------------
// set a CAN message filter
// cf has to be filled with valid data
// result is true, if the filter setting was successful
{
	bool r = false;
	TAcknowledge a;
	cf->cmd = USB_CAN_FILTER;
	if (FOpened) {
		r = Write((char *)cf,sizeof(TCANFilter),100)==sizeof(TCANFilter);
		if (r) {
			r = Read((char *)&a,sizeof(a),100)==sizeof(a);
			if (r) r = a.ack==ACK_OK;  // evaluation of the device response
		}
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TUCI::CAN_SetFilterAll(void)
//---------------------------------------------------------------------------
// this command will set filter bank 0 to pass all messages
// (also power on default)
// Note: After this command filter bank 0 has to be rewritten, if messages
// has to be filtered out
{
	bool r = false;
	TAcknowledge a;
	TCommand Command = {USB_CAN_FILTER_ALL, 0};
	if (FOpened) {
		r = Write((char *)&Command,sizeof(TCommand),10)==sizeof(TCommand);
		if (r) {
			r = Read((char *)&a,sizeof(a),100)==sizeof(a);
			if (r) r = a.ack==ACK_OK;  // evaluation of the device response
		}
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TUCI::Test(void)
//---------------------------------------------------------------------------
// for test purposes (development)
// do not use!
{
	if (FOpened) {
		TCommand Command = {USB_TEST, 0};
		Write((char *)&Command,sizeof(Command),10);
	}
}

